"""
    LDIFAttributeParser.py
    $Author: stevray $
    $Date: 2002/02/13 18:24:06 $

Copyright (C) 2002 Association of Universities for Research in
Astronomy (AURA).  See LICENSE.txt for the usual disclaimers.

"""

__version__ = '$Revision: 1.5 $'[11:-1]

#   The general form of an LDIF attribute definition is as follows:
#
#   ( oid NAME 'objectClassName' DESC 'object class description'
#     EQUALITY matching-rule SYNTAX oid SINGLE-VALUE
#     X-ORIGIN 'who defined this' )
#
#   Each field is as follows:
#
#       oid  - required.  A string of the form n.m.x.y.z...
#       NAME - required.  Always surrounded with single quotes.  May be
#                         a list of aliases.
#       DESC - optional.  If it is given, the value is surrounded
#                         by single quotes.
#       EQUALITY - optional.  Defines the rule for equality for
#                         this 
#       SYNTAX - required. OID of the syntax definition that this
#                         attribute follows.
#                         Defines the class that this class
#                         subclasses.  The superclass name is not
#                         enclosed in quotes.
#       SINGLE-VALUE - optional.  If present, this attribute can
#                         have only a single value; otherwise it
#                         may have multiple values.
#       X-ORIGIN - optional.  If given, the value is surreounded
#                         iin single quotes.

from string import join
from STLDAPUtilities import TRUE, FALSE

class LDIFAttributeParser:

    def __init__( self, LDIFExpression = None):

        if LDIFExpression:

            self._parseLDIF( LDIFExpression )

    def getLDIF(self):
        """
            Returns the LDIF string defining the objectClass.
        """
        oid = self.oid

        if self.aliases:
            name = " NAME ( '" + self.name + "'"
            for alias in self.aliases:
                name = name + " '" + alias + "'"
            name = name + " ) "
        else:
            name = " NAME '" + self.name + "' "

        if self.description:
            desc = "DESC '" + self.description + "' "
        else:
            desc = ""

        if self.matchingRule:
            matchingRule = "EQUALITY " + self.matchingRule + " "
        else:
            matchingRule = ""

        syntax = "SYNTAX " + self.syntax + " "            

        if self.singleValued:
            single = "SINGLE-VALUE "
        else:
            single = ""

        if self.origin:
            origin = "X-ORIGIN '" + self.origin + "'"

        return "( " + oid + name + desc + matchingRule + syntax + single + origin + " )"
        
        
    def _parseLDIF( self, LDIFExpression ):
        
        # Strip surrounding parentheses if they are there.

        if LDIFExpression[0:2] == '( ':
            ldif = LDIFExpression[2:-2]
        else:
            ldif = LDIFExpression

        i = ldif.find(' ')
        self.oid = ldif[0:i]

        self.aliases = self._getAliases(ldif)
        self.name = self.aliases[0]
        self.aliases.remove(self.aliases[0])
        
        i = ldif.find("DESC '")
        if i != -1:
            i = ldif.find("'", i)
            j = ldif.find("'", i + 1)
            self.description = ldif[i+1:j]
        else:
            self.description = ""

        i = ldif.find("EQUALITY ")
        if i != -1:
            i = ldif.find(" ", i)
            j = ldif.find(" ", i + 1)
            self.matchingRule = ldif[i+1:j]
        else:
            self.matchingRule = ""

        i = ldif.find("SYNTAX ")
        if i != -1:
            i = ldif.find(" ", i)
            j = ldif.find(" ", i + 1)
            self.syntax = ldif[i+1:j]
        else:
            self.syntax = ""

        if ldif.find(" SINGLE-VALUE " ) != -1:
            self.singleValued = TRUE
        else:
            self.singleValued = FALSE
        
        i = ldif.find(" X-ORIGIN " )
        if i != -1:
            i = ldif.find( "'", i)
            j = ldif.find( "'", i+1)
            self.origin = ldif[i+1 : j]
        else:
            self.origin = ''

    def _getAliases( self, ldif):
        
        i = ldif.find("NAME")
        if i == -1:
            return []
        i = ldif.find(" ", i+1)
        if ldif[i+1:i+2] == "(":
            i = i + 2
            j = ldif.find( " )", i)
        else:
            j = ldif.find( " ", i+1 )
        tmp = ldif[i+1 : j].replace("'", "")
        return tmp.split(" ")